import { isDevMode, ɵɵdefineInjectable, ɵɵinject, NgZone, Injectable, EventEmitter, Directive, ElementRef, Input, Output, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

class KeyboardShortcutListener {
    constructor(listenerConstructorObject, blackListedKeyboardShortcutChecker) {
        // assign properties based on constructor and defaults
        let defaultListenerOptions = {
            ignoreEventsFromInputElement: false,
            passToLowerPriorities: true,
            priority: 0
        };
        Object.assign(this, defaultListenerOptions, listenerConstructorObject);
        // assign mapped keyboard shortcut combination
        this.mappedKeyboardShortcutCombination = mapKeyboardShortcutCombination(this.keyBinding);
        // assign display shortcut combination
        const copyOfKeyBinding = [...this.keyBinding];
        copyOfKeyBinding.sort((a, b) => {
            if (a.length > 1 && b.length === 1) {
                return -1;
            }
            else if (a.length === 1 && b.length > 1) {
                return 1;
            }
            else if (a > b) {
                return 1;
            }
            else if (a < b) {
                return -1;
            }
            else {
                return 0;
            }
        });
        this.displayShortcutCombination = copyOfKeyBinding
            .map((kb) => kb.replace(/^\w/, (c) => c.toUpperCase()))
            .join(' + ');
        // check vs blacklist
        if (blackListedKeyboardShortcutChecker) {
            blackListedKeyboardShortcutChecker.check(this);
        }
    }
}
/**
 * @enum this allows you to map special keys into your Keyboard Combination
 */
var KeyboardKeys;
(function (KeyboardKeys) {
    KeyboardKeys["Ctrl"] = "ctrl";
    KeyboardKeys["Alt"] = "alt";
    KeyboardKeys["Shift"] = "shift";
    KeyboardKeys["Escape"] = "escape";
})(KeyboardKeys || (KeyboardKeys = {}));
/**
 * @param bindings takes in the shortcut from KeyboardShortcutCombination which is an array of strings
 * @return {string}
 */
function mapKeyboardShortcutCombination(bindings) {
    return JSON.stringify(bindings.map((key) => key.toLowerCase()).sort());
}
// Map to normalized keys across different browser implementations.
// https://github.com/angular/angular/blob/5.0.5/packages/platform-browser/src/browser/browser_adapter.ts#L25-L42
// tslint:disable:object-literal-sort-keys
/**
 * @constant
 * Map to convert some key or keyIdentifier values to what will be returned by getEventKey
 * This is also used to help browser compatibility and to match the W3C standard
 */
const KEY_MAP = {
    '\b': 'backspace',
    '\t': 'tab',
    '\x1B': 'escape',
    '\x7F': 'delete',
    Del: 'delete',
    Esc: 'escape',
    Left: 'arrowleft',
    Right: 'arrowright',
    Up: 'arrowup',
    Down: 'arrowdown',
    Menu: 'contextMenu',
    Scroll: 'scrolllock',
    Win: 'os',
    ' ': 'space',
    '.': 'dot'
};
// tslint:enable:object-literal-sort-keys

const blacklistedKeyCombinations = [
    [KeyboardKeys.Ctrl, 'd'],
    [KeyboardKeys.Ctrl, 'f'],
    [KeyboardKeys.Ctrl, 'g'],
    [KeyboardKeys.Ctrl, KeyboardKeys.Shift, 'g'],
    [KeyboardKeys.Ctrl, 'h'],
    [KeyboardKeys.Ctrl, 'j'],
    [KeyboardKeys.Ctrl, 'o'],
    [KeyboardKeys.Ctrl, 's'],
    [KeyboardKeys.Ctrl, 'p'],
    [KeyboardKeys.Ctrl, 'u'],
    [KeyboardKeys.Ctrl, 'n'],
    [KeyboardKeys.Ctrl, 't'],
    [KeyboardKeys.Ctrl, KeyboardKeys.Shift, 't'],
    [KeyboardKeys.Ctrl, 'c'],
    [KeyboardKeys.Ctrl, 'x'],
    [KeyboardKeys.Ctrl, 'v'],
    [KeyboardKeys.Alt, 'f'],
    [KeyboardKeys.Ctrl, '0'],
    [KeyboardKeys.Ctrl, '1'],
    [KeyboardKeys.Ctrl, '2'],
    [KeyboardKeys.Ctrl, '3'],
    [KeyboardKeys.Ctrl, '4'],
    [KeyboardKeys.Ctrl, '5'],
    [KeyboardKeys.Ctrl, '6'],
    [KeyboardKeys.Ctrl, '7'],
    [KeyboardKeys.Ctrl, '8'],
    [KeyboardKeys.Ctrl, '9'],
    [KeyboardKeys.Shift, '1'],
    [KeyboardKeys.Shift, '2'],
    [KeyboardKeys.Shift, '3'],
    [KeyboardKeys.Shift, '4'],
    [KeyboardKeys.Shift, '5'],
    [KeyboardKeys.Shift, '6'],
    [KeyboardKeys.Shift, '7'],
    [KeyboardKeys.Shift, '8'],
    [KeyboardKeys.Shift, '9'],
    [KeyboardKeys.Shift, '0'],
    [KeyboardKeys.Escape],
    [KeyboardKeys.Ctrl, 'tab'],
    [KeyboardKeys.Ctrl, KeyboardKeys.Shift, 'tab'],
    [KeyboardKeys.Ctrl, 'w'],
    [KeyboardKeys.Alt, '4'],
    [KeyboardKeys.Alt, 'arrowleft'],
    [KeyboardKeys.Alt, 'backspace'],
    [KeyboardKeys.Ctrl, KeyboardKeys.Shift, 'delete'] // open the clear browsing history window
];
class BlackListedKeyboardShortcutChecker {
    constructor() {
        this.blackListedKeyBindings = blacklistedKeyCombinations;
        this.mappedBlackListedKeyBindings = this.blackListedKeyBindings.map((blkb) => mapKeyboardShortcutCombination(blkb));
    }
    check(listener) {
        // this is where the test was deleted
        for (let blkb of this.mappedBlackListedKeyBindings) {
            if (blkb === listener.mappedKeyboardShortcutCombination) {
                const warningMessage = `Keyboard Shortcut [${listener.displayShortcutCombination}] is blacklisted as a common keyboard shortcut.  Consider changing your keybinding for this shortcut.`;
                console.warn(warningMessage);
                return;
            }
        }
    }
}

class KeyboardShortcutService {
    constructor(zone) {
        this._listeners = [];
        this.zone = zone;
        // Since we're going to create a root event-handler for the keydown event, we're
        // gonna do this outside of the NgZone. This way, we're not constantly triggering
        // change-detection for every key event - we'll only re-enter the Angular Zone
        // when we have an event that is actually being consumed by one of our components.
        this.zone.runOutsideAngular(() => {
            window.addEventListener('keydown', this.handleKeyboardEvent.bind(this));
        });
        // we only use this if isDevMode
        if (isDevMode()) {
            this.blackListedKeyboardShortcutChecker = new BlackListedKeyboardShortcutChecker();
        }
    }
    /**
     * @readonly
     * @description A list of all available shortcut listeners
     * @property    {IKeyboardShortcutListener[]}
     */
    get listeners_read_only() {
        return JSON.parse(JSON.stringify(this._listeners));
    }
    /**
     * @description                     Creates a listenerHandle using imported interface
     * @param listenerConstructorObject Interface that contains a KeyboardShortcutHandler
     * @returns                         an IListenerHandle
     */
    listen(listenerConstructorObject) {
        const listenerHandle = this.addListener(listenerConstructorObject);
        return listenerHandle;
    }
    addListener(listenerConstructorObject) {
        const listener = new KeyboardShortcutListener(listenerConstructorObject, this.blackListedKeyboardShortcutChecker);
        const listenerHandle = {
            remove: () => {
                this.removeListener(listener);
            }
        };
        this._listeners.push(listener);
        this._listeners.sort((a, b) => {
            if (a.priority < b.priority) {
                return 1;
            }
            else if (a.priority > b.priority) {
                return -1;
            }
            else {
                return 0;
            }
        });
        return listenerHandle;
    }
    // get the keyCombination from the given event
    getKeyFromEvent(event) {
        const keyIdentifier = 'keyIdentifier';
        let key = event.key || event[keyIdentifier] || 'Unidentified';
        if (key.startsWith('U+')) {
            key = String.fromCharCode(parseInt(key.slice(2), 16));
        }
        let parts = [(KEY_MAP[key] || key)];
        if (event.altKey) {
            parts.push(KeyboardKeys.Alt);
        }
        if (event.ctrlKey) {
            parts.push(KeyboardKeys.Ctrl);
        }
        if (event.shiftKey) {
            parts.push(KeyboardKeys.Shift);
        }
        return mapKeyboardShortcutCombination(parts);
    }
    // handle the keyboard events for the root handler (and delegate to the listeners).
    handleKeyboardEvent(event) {
        let keyCombination = this.getKeyFromEvent(event);
        let isInputEvent = this.isEventFromInput(event);
        let handler;
        // Iterate over the listeners in DESCENDING priority order.
        for (let listener of this._listeners) {
            if (listener.mappedKeyboardShortcutCombination === keyCombination) {
                handler = listener.handler;
                // Execute handler if this is NOT an input event that we need to ignore.
                if (!isInputEvent || !listener.ignoreEventsFromInputElement) {
                    // Right now, we're executing outside of the NgZone. As such, we
                    // have to re-enter the NgZone so that we can hook back into change-
                    // detection. Plus, this will also catch errors and propagate them
                    // through application properly.
                    this.zone.runGuarded(() => {
                        return handler(event);
                    });
                    // If the handler returned an explicit False, we're going to treat
                    // this listener as Terminal, regardless of the original settings.
                    if (!listener.passToLowerPriorities) {
                        return;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    /**
     * @description Used for testing, Sends a keyboard event
     * @param event a keyboard event
     */
    sendKeyboardEventToHandler(event) {
        this.handleKeyboardEvent(event);
    }
    // determine if the given event originated from a form input element.
    isEventFromInput(event) {
        if (event.target instanceof Node) {
            switch (event.target.nodeName) {
                case 'INPUT':
                case 'SELECT':
                case 'TEXTAREA':
                    return true;
                default:
                    return false;
            }
        }
        return false;
    }
    // remove the given listener from the internal collection.
    removeListener(listenerToRemove) {
        this._listeners = this._listeners.filter((listener) => {
            return listener !== listenerToRemove;
        });
    }
}
KeyboardShortcutService.ɵprov = ɵɵdefineInjectable({ factory: function KeyboardShortcutService_Factory() { return new KeyboardShortcutService(ɵɵinject(NgZone)); }, token: KeyboardShortcutService, providedIn: "root" });
KeyboardShortcutService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
KeyboardShortcutService.ctorParameters = () => [
    { type: NgZone }
];

class KeyboardShortcutDirective {
    constructor(elRef, keyboardShortcutService) {
        this.elRef = elRef;
        this.keyboardShortcutService = keyboardShortcutService;
        this.fireClickEventOnKeyboardShortcut = true;
        this.onKeyboardShortcut = new EventEmitter();
    }
    ngOnInit() {
        if (!this.keyboardShortcut) {
            return;
        }
        let listenerConstructor = {};
        Object.assign(listenerConstructor, { handler: this.keyboardShortcutHandler.bind(this) }, this.keyboardShortcut);
        this.listener = this.keyboardShortcutService.listen(listenerConstructor);
    }
    keyboardShortcutHandler(event) {
        if (this.fireClickEventOnKeyboardShortcut) {
            if (this.elRef &&
                this.elRef.nativeElement &&
                this.elRef.nativeElement.click) {
                this.elRef.nativeElement.click();
            }
        }
        this.onKeyboardShortcut.emit(event);
    }
    ngOnDestroy() {
        if (this.listener) {
            this.listener.remove();
        }
    }
}
KeyboardShortcutDirective.decorators = [
    { type: Directive, args: [{
                selector: '[keyboardShortcut]'
            },] }
];
KeyboardShortcutDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: KeyboardShortcutService }
];
KeyboardShortcutDirective.propDecorators = {
    keyboardShortcut: [{ type: Input }],
    fireClickEventOnKeyboardShortcut: [{ type: Input }],
    onKeyboardShortcut: [{ type: Output }]
};

class NgxKeyboardShortcutModule {
}
NgxKeyboardShortcutModule.decorators = [
    { type: NgModule, args: [{
                declarations: [KeyboardShortcutDirective],
                exports: [KeyboardShortcutDirective],
                imports: [CommonModule]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { KEY_MAP, KeyboardKeys, KeyboardShortcutDirective, KeyboardShortcutListener, KeyboardShortcutService, NgxKeyboardShortcutModule, blacklistedKeyCombinations, mapKeyboardShortcutCombination, BlackListedKeyboardShortcutChecker as ɵa };
//# sourceMappingURL=ngx-keyboard-shortcuts.js.map
